// ignore_for_file: cancel_subscriptions, public_member_api_docs, non_constant_identifier_names,lines_longer_than_80_chars, empty_catches, avoid_dynamic_calls

import 'dart:developer';
import 'dart:io';

import 'package:flutter_blue_plus/flutter_blue_plus.dart';
import 'package:healthrian_ble_support/healthrian_ble_support.dart';
import 'package:healthrian_ble_support_for_cardioclick/'
    'healthrian_ble_support_for_cardioclick.dart' as cardioclick;
import 'package:healthrian_ble_support_for_wearcardiot/'
    'healthrian_ble_support_for_wearcardiot.dart' as wearcardiot;
import 'package:healthrian_ble_support_for_wearecg12/'
    'healthrian_ble_support_for_wearecg12.dart' as wearecg12;
import 'package:healthrian_common_support/healthrian_common_support.dart';

class BLEManager {
  factory BLEManager() {
    return _instance;
  }

  BLEManager._internal() {
    raw.isAvailable.then(
      (isAvailable) {
        if (isAvailable) raw.setLogLevel(LogLevel.info);
      },
    );
  }

  static final BLEManager _instance = BLEManager._internal();
  final FlutterBluePlus raw = FlutterBluePlus.instance;

  final manageWearEcg12 = wearecg12.BLEManager();
  final manageCardioClick = cardioclick.BLEManager();
  final manageWearCardioT = wearcardiot.BLEManager();

  final _enable = List<bool>.generate(3, (index) => true);

  void setMode({
    List<bool> enableList = const [true, true, true],
  }) {
    assert(enableList.length >= 3, 'Length of input list must be 3');
    for (var i = 0; i < enableList.length; i++) {
      _enable[i] = enableList[i];
    }
  }

  void enableWearEcg12({bool enable = true}) {
    setMode(enableList: [true, false, false]);
  }

  void enableCardioClick({bool enable = true}) {
    setMode(enableList: [false, true, false]);
  }

  void enableWearCardioT({bool enable = true}) {
    setMode(enableList: [false, false, true]);
  }

  // Future<bool> connectDevice({String preferredDeviceId = ''}) async {
  //   if (!_enable.contains(true)) return false;
  //   return _perform(
  //     () => manageWearEcg12.connectDevice(
  //       preferredDeviceId: preferredDeviceId,
  //     ),
  //     () => manageCardioClick.connectDevice(
  //       preferredDeviceId: preferredDeviceId,
  //     ),
  //     () => manageWearCardioT.connectDevice(
  //       preferredDeviceId: preferredDeviceId,
  //     ),
  //     isDeviceCheck: false,
  //   )!;
  // }

  BluetoothDevice? device;

  final FlutterBluePlus _flutterBlue = FlutterBluePlus.instance;

  Future<bool> connectDevice({String preferredDeviceId = ''}) async {
    // 0. 초기화
    device = null;

    log(_enable.toString());

    final scanFilters = [
      if (_enable[0]) manageWearEcg12.scanFilter,
      if (_enable[1]) manageCardioClick.scanFilter,
      if (_enable[2]) manageWearCardioT.scanFilter,
    ];

    setManager(device) {
      if (_enable[0] && manageWearEcg12.scanFilter(device))
        manageWearEcg12.device = device;
      if (_enable[1] && manageCardioClick.scanFilter(device))
        manageCardioClick.device = device;
      if (_enable[2] && manageWearCardioT.scanFilter(device))
        manageWearCardioT.device = device;
    }
    log(scanFilters.toString());

    // 1. 연결 장비 확인
    final connectedDevices = await _flutterBlue.connectedDevices;
    final connectedFeasibleDevices = connectedDevices.where(
      (e) => scanFilters.where((func) => func(e)).contains(true),
    );
    log(connectedFeasibleDevices.toString());
    if (connectedFeasibleDevices.isNotEmpty) {
      device = connectedFeasibleDevices.firstWhere(
        (device) => device.id.toString() == preferredDeviceId,
        orElse: () => connectedFeasibleDevices.first,
      );
      setManager(device);
      return true;
    }

    // 2. 스캔 수행
    final scan = _flutterBlue.startScan(
      timeout: const Duration(seconds: 2),
    );
    final isFoundFeasibleDevice = await scan.then(
      (events) async {
        final scanResults = events as List<ScanResult>;
        final feasibleScanResult = scanResults
            // .where(
            //   (result) => scanFilters
            //       .where((func) => func(result.device))
            //       .contains(true),
            // )
            .toList(growable: false)
          ..sort(
            (a, b) => b.rssi - a.rssi,
          );
        log(feasibleScanResult.toString());

        if (feasibleScanResult.isNotEmpty) {
          final result = feasibleScanResult.firstWhere(
            (r) => r.device.id.toString() == preferredDeviceId,
            orElse: () => feasibleScanResult.first,
          );
          device = result.device;
          if (Platform.isAndroid) {
            await device!.pair();
          }
          log(device.toString());
          await device!.connect();
          setManager(device);
          return true;
        }
        return false;
      },
    );
    return isFoundFeasibleDevice;
  }

  bool _isAbleToWearCardioT() => manageWearCardioT.device != null && _enable[2];

  bool _isAbleToCardioClick() => manageCardioClick.device != null && _enable[1];

  bool _isAbleToWearEcg12() => manageWearEcg12.device != null && _enable[0];

  // bool get _isMultiSelectionMode => _enable.where((e) => e).length > 1;

  // T? _performSequential<T>(
  //   T Function() functionWearEcg12,
  //   T Function() functionCardioClick,
  //   T Function() functionWearCardioT, {
  //   bool isDeviceCheck = true,
  // }) {
  //   T? res;
  //
  //   // functionWearEcg12
  //
  //   return res;
  // }

  T? _perform<T>(
    T Function() functionWearEcg12,
    T Function() functionCardioClick,
    T Function() functionWearCardioT,
    //     {
    //   bool isDeviceCheck = true,
    // }
  ) {
    // if (_isMultiSelectionMode) {
    //   return _performSequential(
    //     functionWearEcg12,
    //     functionCardioClick,
    //     functionWearCardioT,
    //   );
    // }
    if (_isAbleToWearEcg12()) {
      return functionWearEcg12();
    }
    if (_isAbleToCardioClick()) {
      return functionCardioClick();
    }
    if (_isAbleToWearCardioT()) {
      return functionWearCardioT();
    }
    return null;
  }

  Future<void> disconnectDevice() async {
    await _perform(
      manageWearEcg12.disconnectDevice,
      manageCardioClick.disconnectDevice,
      manageWearCardioT.disconnectDevice,
    );
  }

  Future<void> registerCharacteristics() async {
    await _perform(
      manageWearEcg12.registerCharacteristics,
      manageCardioClick.registerCharacteristics,
      manageWearCardioT.registerCharacteristics,
    );
  }

  Future<void> initializeStreams() async {
    await _perform(
      manageWearEcg12.initializeStreams,
      manageCardioClick.initializeStreams,
      manageWearCardioT.initializeStreams,
    );
  }

  Future<void> startMeasurement() async {
    await _perform(
      manageWearEcg12.startMeasurementEcg,
      manageCardioClick.startMeasurementEcg,
      manageWearCardioT.startMeasurement,
    );
  }

  Future<void> stopMeasurement() async {
    await _perform(
      manageWearEcg12.stopMeasurementEcg,
      manageCardioClick.stopMeasurementEcg,
      manageWearCardioT.stopMeasurement,
    );
  }

  Future<void> pauseMeasurement() async {
    await _perform(
      manageWearEcg12.pauseMeasurementEcg,
      manageCardioClick.pauseMeasurementEcg,
      manageWearCardioT.pauseMeasurement,
    );
  }

  int get samplingRate {
    return _perform(
          () => manageWearEcg12.samplingRate,
          () => manageCardioClick.samplingRate,
          () => manageWearCardioT.samplingRate,
        ) ??
        250;
  }

  HolterDevices? getDevice() {
    return _perform(
      () => HolterDevices.WearECG12,
      () => HolterDevices.CardioClick,
      () => HolterDevices.WearCardioT,
    );
  }

  Future<Channels?> saveMeasurement() async {
    final res = _perform(
      () async {
        final channels = manageWearEcg12.lastChannel;
        if (channels == null) return null;
        return Channels.fromList(
          channels.toList(),
          manageWearEcg12.dataFilter,
        );
      },
      () async {
        final channels = manageCardioClick.lastChannel;
        if (channels == null) return null;
        return Channels.fromList(
          channels.toList(),
          manageCardioClick.dataFilter,
        );
      },
      () async {
        final channels = manageWearCardioT.lastChannelEcg;
        if (channels == null) return null;
        return Channels.fromList(
          channels.toList(),
          manageWearCardioT.dataFilter,
        );
      },
    );
    return res;
  }
}
